"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var Platform = require('platform');
var absolute_layout_1 = require('ui/layouts/absolute-layout');
var stack_layout_1 = require('ui/layouts/stack-layout');
var view_1 = require('ui/core/view');
var button_1 = require('ui/button');
var utilities_1 = require('./utilities');
var AnimationModule = require('ui/animation');
var gestures = require('ui/gestures');
var Slide = (function (_super) {
    __extends(Slide, _super);
    function Slide() {
        _super.apply(this, arguments);
    }
    return Slide;
}(stack_layout_1.StackLayout));
exports.Slide = Slide;
var direction;
(function (direction) {
    direction[direction["none"] = 0] = "none";
    direction[direction["left"] = 1] = "left";
    direction[direction["right"] = 2] = "right";
})(direction || (direction = {}));
var IntroSlides = (function (_super) {
    __extends(IntroSlides, _super);
    function IntroSlides() {
        var _this = this;
        _super.call(this);
        this.direction = direction.none;
        this._addChildFromBuilder = function (name, value) {
            if (value instanceof view_1.View) {
                if (value instanceof Slide) {
                    _this._childSlides.push(value);
                }
                _this.addChild(value);
            }
        };
        this.constructView();
    }
    Object.defineProperty(IntroSlides.prototype, "btnNext", {
        get: function () {
            return this._btnNext;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IntroSlides.prototype, "btnPrevious", {
        get: function () {
            return this._btnPrevious;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IntroSlides.prototype, "pageWidth", {
        get: function () {
            return this._pageWidth;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IntroSlides.prototype, "android", {
        get: function () {
            return;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IntroSlides.prototype, "ios", {
        get: function () {
            return;
        },
        enumerable: true,
        configurable: true
    });
    IntroSlides.prototype.constructView = function () {
        var _this = this;
        this._loaded = false;
        this.transitioning = false;
        this._childSlides = [];
        this._pageWidth = Platform.screen.mainScreen.widthDIPs;
        var pageHeight = Platform.screen.mainScreen.heightDIPs;
        this.on(absolute_layout_1.AbsoluteLayout.loadedEvent, function (data) {
            if (!_this._loaded) {
                _this._loaded = true;
                var footer = _this.buildFooter();
                _this._btnPrevious = footer.getViewById('btn-info-previous');
                _this._btnNext = footer.getViewById('btn-info-next');
                var info = footer.getViewById('info');
                var slides_1 = [];
                _this.eachLayoutChild(function (view) {
                    if (view instanceof stack_layout_1.StackLayout) {
                        absolute_layout_1.AbsoluteLayout.setLeft(view, _this.pageWidth);
                        view.width = _this.pageWidth;
                        view.height = '100%';
                        slides_1.push(view);
                    }
                });
                _this.currentPanel = utilities_1.SlideUtilities.buildSlideMap(slides_1);
                _this.btnNext.on('tap', function () {
                    _this.transitioning = true;
                    _this.showRightSlide(_this.currentPanel).then(function () {
                        console.log('right button done');
                        _this.setupRightPanel();
                    });
                });
                _this.btnPrevious.on('tap', function () {
                    _this.transitioning = true;
                    _this.showLeftSlide(_this.currentPanel).then(function () {
                        _this.setupLeftPanel();
                    });
                });
                if (isNaN(footer.height)) {
                    footer.height = 76;
                }
                absolute_layout_1.AbsoluteLayout.setTop(footer, (pageHeight - footer.height));
                _this.currentPanel.panel.translateX = -_this.pageWidth;
                _this.btnPrevious.visibility = 'collapse';
                _this.applySwipe(_this.pageWidth);
            }
        });
    };
    IntroSlides.prototype.setupLeftPanel = function () {
        this.direction = direction.none;
        this.transitioning = false;
        this.btnNext.visibility = 'visible';
        this.btnPrevious.visibility = 'visible';
        this.currentPanel.panel.off('touch,pan');
        this.currentPanel = this.currentPanel.left;
        this.applySwipe(this.pageWidth);
        if (this.currentPanel.left == null) {
            this.btnPrevious.visibility = 'collapse';
        }
    };
    IntroSlides.prototype.setupRightPanel = function () {
        this.direction = direction.none;
        this.transitioning = false;
        this.btnNext.visibility = 'visible';
        this.btnPrevious.visibility = 'visible';
        this.currentPanel.panel.off('touch,pan');
        this.currentPanel = this.currentPanel.right;
        this.applySwipe(this.pageWidth);
        if (this.currentPanel.right == null) {
            this.btnNext.visibility = 'collapse';
        }
    };
    IntroSlides.prototype.applySwipe = function (pageWidth) {
        var _this = this;
        var previousDelta = -1;
        this.currentPanel.panel.on('pan', function (args) {
            if (args.state === gestures.GestureStateTypes.ended) {
                if (_this.transitioning === false) {
                    _this.transitioning = true;
                    _this.currentPanel.panel.translateX = -_this.pageWidth;
                    if (_this.currentPanel.right != null) {
                        _this.currentPanel.right.panel.translateX = -_this.pageWidth;
                        _this.currentPanel.right.panel.translateX = _this.pageWidth;
                    }
                    _this.transitioning = false;
                }
            }
            else {
                if (!_this.transitioning && previousDelta !== args.deltaX && args.deltaX != null && args.deltaX < -5) {
                    if (_this.currentPanel.right != null) {
                        _this.direction = direction.left;
                        _this.currentPanel.panel.translateX = args.deltaX - _this.pageWidth;
                        _this.currentPanel.right.panel.translateX = args.deltaX;
                        if (args.deltaX < ((pageWidth / 3) * -1)) {
                            _this.transitioning = true;
                            _this.showRightSlide(_this.currentPanel, args.deltaX).then(function () {
                                _this.setupRightPanel();
                            });
                            ;
                        }
                    }
                }
                if (!_this.transitioning && previousDelta !== args.deltaX && args.deltaX != null && args.deltaX > 5) {
                    if (_this.currentPanel.left != null) {
                        _this.direction = direction.right;
                        _this.currentPanel.panel.translateX = args.deltaX - _this.pageWidth;
                        _this.currentPanel.left.panel.translateX = -(_this.pageWidth * 2) + args.deltaX;
                        if (args.deltaX > pageWidth / 3) {
                            _this.transitioning = true;
                            _this.showLeftSlide(_this.currentPanel, args.deltaX).then(function () {
                                _this.setupLeftPanel();
                            });
                        }
                    }
                }
                if (args.deltaX !== 0) {
                    previousDelta = args.deltaX;
                }
            }
        });
    };
    IntroSlides.prototype.showRightSlide = function (panelMap, offset) {
        if (offset === void 0) { offset = 0; }
        var transition = new Array();
        transition.push({
            target: panelMap.right.panel,
            translate: { x: -this.pageWidth, y: 0 },
            duration: 300,
        });
        transition.push({
            target: panelMap.panel,
            translate: { x: -this.pageWidth * 2, y: 0 },
            duration: 300,
        });
        var animationSet = new AnimationModule.Animation(transition, false);
        return animationSet.play();
    };
    IntroSlides.prototype.showLeftSlide = function (panelMap, offset) {
        if (offset === void 0) { offset = 0; }
        var transition = new Array();
        transition.push({
            target: panelMap.left.panel,
            translate: { x: -this.pageWidth, y: 0 },
            duration: 500,
        });
        transition.push({
            target: panelMap.panel,
            translate: { x: 0, y: 0 },
            duration: 500,
        });
        var animationSet = new AnimationModule.Animation(transition, false);
        return animationSet.play();
    };
    IntroSlides.prototype.buildFooter = function () {
        var footer = new absolute_layout_1.AbsoluteLayout();
        var footerInnerWrap = new stack_layout_1.StackLayout();
        var footerInnerWrapLeft = new stack_layout_1.StackLayout();
        var footerInnerWrapMiddle = new stack_layout_1.StackLayout();
        var footerInnerWrapRight = new stack_layout_1.StackLayout();
        this.setwidthPercent(footer, 100);
        footerInnerWrap.orientation = 'horizontal';
        this.setwidthPercent(footerInnerWrap, 100);
        footer.addChild(footerInnerWrap);
        this.setwidthPercent(footerInnerWrapRight, 30);
        this.setwidthPercent(footerInnerWrapLeft, 30);
        this.setwidthPercent(footerInnerWrapMiddle, 40);
        footerInnerWrapLeft.addChild(this.newFooterButton('Previous'));
        footerInnerWrapRight.addChild(this.newFooterButton('Next'));
        footerInnerWrap.addChild(footerInnerWrapLeft);
        footerInnerWrap.addChild(footerInnerWrapMiddle);
        footerInnerWrap.addChild(footerInnerWrapRight);
        return footer;
    };
    IntroSlides.prototype.setwidthPercent = function (view, percentage) {
        view.width = percentage + '%';
    };
    IntroSlides.prototype.newFooterButton = function (name) {
        var button = new button_1.Button();
        button.id = 'btn-info-' + name.toLowerCase();
        button.text = name;
        this.setwidthPercent(button, 100);
        return button;
    };
    return IntroSlides;
}(absolute_layout_1.AbsoluteLayout));
exports.IntroSlides = IntroSlides;
//# sourceMappingURL=nativescript-intro-slides.js.map